import { isReservedEvent, isReservedProperty } from './reserved.js';
export function createElementMetadata(customElementsManifest, entrypoint) {
    const modules = getCustomElementModules(customElementsManifest);
    const elements = modules.flatMap(m => {
        return m.declarations.filter(d => d.customElement && d.tagName).map(d => {
            var _a, _b, _c, _d;
            const path = `${entrypoint ? `${entrypoint}` : './'}${replaceTsExtentions(m.path)}`;
            const element = {
                name: d.name,
                tagName: d.tagName,
                description: (_a = d.description) !== null && _a !== void 0 ? _a : '',
                path,
                import: `import { ${d.name} } from '${path}';`,
                importType: `import type { ${d.name} } from '${path}';`,
                slots: (_b = d.slots) !== null && _b !== void 0 ? _b : [],
                cssProperties: (_c = d.cssProperties) !== null && _c !== void 0 ? _c : [],
                events: (_d = getCustomElementEvents(d)) !== null && _d !== void 0 ? _d : [],
                propeties: getPublicProperties(d)
            };
            return element;
        });
    });
    return elements;
}
function replaceTsExtentions(filePath) {
    return filePath.endsWith('.ts') ? changeExt(filePath, 'js') : filePath;
}
function changeExt(filePath, ext) {
    const pos = filePath.includes('.') ? filePath.lastIndexOf('.') : filePath.length;
    return `${filePath.substr(0, pos)}.${ext}`;
}
function getPublicProperties(element) {
    var _a, _b;
    return ((_b = (_a = element.members) === null || _a === void 0 ? void 0 : _a.filter(m => !m.readonly &&
        !m.static &&
        m.kind === 'field' &&
        m.attribute !== undefined &&
        m.privacy === undefined &&
        m.privacy !== 'private' &&
        m.privacy !== 'protected' &&
        m.name !== 'accessor' &&
        !isReservedProperty(m.name))) !== null && _b !== void 0 ? _b : []).map(p => { var _a; return ({ name: p.name, type: (_a = p.type) === null || _a === void 0 ? void 0 : _a.text }); });
}
function getCustomElementModules(customElementsManifest) {
    return customElementsManifest.modules.filter(m => { var _a; return ((_a = m.declarations) === null || _a === void 0 ? void 0 : _a.length) && m.declarations.find(d => d.customElement === true); });
}
function getCustomElementEvents(element) {
    var _a;
    const memberEvents = element.members
        .filter(event => event.privacy === undefined) // public
        .filter(prop => { var _a, _b; return prop.type && ((_a = prop.type) === null || _a === void 0 ? void 0 : _a.text) && ((_b = prop.type) === null || _b === void 0 ? void 0 : _b.text.includes('EventEmitter')) && !isReservedEvent(prop.name); })
        .map(event => ({ name: event.name }));
    const events = (_a = element.events) !== null && _a !== void 0 ? _a : [];
    return Object.values(Object.values([...memberEvents, ...events].reduce((prev, next) => ({ ...prev, [next.name]: next }), {})));
}
export const generatedMessage = `Generated with https://github.com/blueprintui/custom-element-types`;
